import pytest

from articles.models import Article, Category
from articles.forms import ArticleForm


@pytest.fixture
def category():
    category = Category()
    category.name = 'name1'

    return category


@pytest.fixture
def article():
    article = Article()
    article.title = 'title1'
    article.description = 'description1'
    article.updated_at = 'updated_at1'
    article.category = category()
    article.published = 'published1'

    return article


@pytest.fixture
def article_form():
    article_form = ArticleForm()
    article_form.title = 'title'
    article_form.description = 'description'
    article_form.updated_at = 'updated_at'
    article_form.category = category()
    article_form.published = 'published'

    return article_form


def get_form(form):
    """
    This is a hack to make sure get_form returns what you expect
    the Django way. I don't know why get_form does not work yet.
    """
    def _get_form():
            return {field.name: field
                    for field in form}
    return _get_form



from collections import OrderedDict

from django import forms
from django.contrib.auth import forms as user_forms
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext as _

from betterforms.multiform import MultiModelForm

from arctic.loading import get_user_role_model

User = get_user_model()
UserRole = get_user_role_model()


class UserRoleForm(forms.ModelForm):
    class Meta:
        model = UserRole
        fields = ('role',)


class UserCreationForm(user_forms.UserCreationForm):
    class Meta:
        model = User
        fields = ('username', 'email', 'is_active',)


class UserCreationMultiForm(MultiModelForm):
    form_classes = OrderedDict([
        ('user', UserCreationForm),
        ('role', UserRoleForm),
    ])

    def save(self, commit=True):
        objects = super(UserCreationMultiForm, self).save(commit=False)

        if commit:
            user = objects['user']
            user.save()
            user_role = objects['role']
            user_role.user = user
            user_role.save()

        return objects


class UserChangeForm(forms.ModelForm):
    new_password = forms.CharField(
        label=_("New Password"),
        widget=forms.PasswordInput,
        required=False,
        help_text=_("Leave this field empty if you don't want to change your "
                    "password."),
    )

    class Meta:
        model = User
        fields = ('username', 'email', 'is_active',)

    def save(self, commit=True):
        user = super(UserChangeForm, self).save(commit=False)

        new_password = self.cleaned_data["new_password"]
        if new_password:
            user.set_password(new_password)
        if commit:
            user.save()
        return user


class UserChangeMultiForm(MultiModelForm):
    form_classes = OrderedDict([
        ('user', UserChangeForm),
        ('role', UserRoleForm),
    ])



from __future__ import (absolute_import, unicode_literals)

from django import forms

from .models import Article


class ArticleForm(forms.ModelForm):
    class Meta:
        fields = ['title', 'description', 'category', 'updated_at', 'tags',
                  'published']
        model = Article

    def __init__(self, *args, **kwargs):
        super(ArticleForm, self).__init__(*args, **kwargs)
        self.fields['category'].empty_label = ""



from __future__ import (division, unicode_literals)

from collections import OrderedDict

from django.conf import settings
from django.contrib.auth import (authenticate, login, logout)
from django.core.exceptions import (FieldDoesNotExist)
from django.core.urlresolvers import (NoReverseMatch, reverse)
from django.db.models.deletion import (Collector, ProtectedError)
from django.shortcuts import (redirect, render, resolve_url)
from django.utils.http import quote
from django.utils.text import capfirst
from django.utils.translation import ugettext as _
from django.views import generic as base

import extra_views

from .filters import filterset_factory
from .mixins import (LinksMixin, RoleAuthentication, SuccessMessageMixin,
                     LayoutMixin)
from .utils import (find_attribute, find_field_meta, get_attribute, menu,
                    view_from_url)


class View(RoleAuthentication, base.View):
    """
    This view needs to be used for all Arctic views, except the LoginView.

    It includes integration with the Arctic user interface elements, such as
    the menu, site logo, site title, page title and breadcrumbs.
    """

    page_title = ''
    page_description = ''
    breadcrumbs = None
    tabs = None
    requires_login = True
    urls = {}

    def dispatch(self, request, *args, **kwargs):
        """
        Most views in a CMS require a login, so this is the default setup.

        If a login is not required then the requires_login property
        can be set to False to disable this.
        """
        if (not request.user.is_authenticated()) and self.requires_login:
            return redirect('%s?next=%s' % (resolve_url(settings.LOGIN_URL),
                                            quote(request.get_full_path())))
        return super(View, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(View, self).get_context_data(**kwargs)
        context['page_title'] = self.get_page_title()
        context['page_description'] = self.get_page_description()
        context['menu'] = menu(user=self.request.user, request=self.request)
        context['urls'] = self.get_urls()
        context['breadcrumbs'] = self.get_breadcrumbs()
        context['tabs'] = self.get_tabs()
        context['index_url'] = self.get_index_url()
        context['SITE_NAME'] = self.get_site_name()
        context['SITE_TITLE'] = self.get_site_title()
        context['SITE_LOGO'] = self.get_site_logo()
        context['TOPBAR_BACKGROUND_COLOR'] = self.get_topbar_background_color()
        context['HIGHLIGHT_COLOR'] = self.get_highlight_color()
        context['DATETIME_FORMATS'] = self.get_datetime_formats()
        return context

    def get_urls(self):
        """
        Used for resolving urls when displaying nested objects, see arctic_url.

        For example, generally you just have /foo/create as
        a url, but with nested, you may have: /foo/<id>/bar/create/ and <id>
        would be a parent id. These are then required to resolve urls.

        @returns
        {named_url, (url_param, url_param),}
        || {named_url, [url_param, url_param],}

        if you provide a list and in this list there are strings, it will try
        to get field of that item. This is especially useful for listviews with
        action_links and field_links.
        """
        return self.urls

    def get_breadcrumbs(self):
        """
        Breadcrumb format: (('name', 'url'), ...) or None if not used.
        """
        if not self.breadcrumbs:
            return None
        else:
            allowed_breadcrumbs = []
            for breadcrumb in self.breadcrumbs:

                # check permission based on named_url
                if breadcrumb[1] is not None \
                        and not view_from_url(
                            breadcrumb[1]).has_permission(self.request.user):
                    continue

                allowed_breadcrumbs.append(breadcrumb)
            return allowed_breadcrumbs

    def get_tabs(self):
        """
        Tabs format: (('name', 'url'), ...) or None if tabs are not used.
        """
        if not self.tabs:
            return None
        else:
            allowed_tabs = []
            for tab in self.tabs:

                # check permission based on named_url
                if not view_from_url(tab[1]).has_permission(self.request.user):
                    continue

                allowed_tabs.append(tab)
            return allowed_tabs

    def get_page_title(self):
        return self.page_title

    def get_page_description(self):
        return self.page_description

    def get_site_logo(self):
        return getattr(settings, 'ARCTIC_SITE_LOGO',
                       'arctic/dist/assets/img/arctic_logo.svg')

    def get_site_name(self):
        return getattr(settings, 'ARCTIC_SITE_NAME',
                       'Arctic Site Name')

    def get_site_title(self):
        return getattr(settings, 'ARCTIC_SITE_TITLE',
                       self.get_site_name())

    def get_topbar_background_color(self):
        return getattr(settings, 'ARCTIC_TOPBAR_BACKGROUND_COLOR', None)

    def get_highlight_color(self):
        return getattr(settings, 'ARCTIC_HIGHLIGHT_COLOR', None)

    def get_index_url(self):
        try:
            return reverse('index')
        except NoReverseMatch:
            return '/'

    def get_datetime_formats(self):
        dtformats = {}

        dtformats['SHORT_DATE'] = settings.DATE_INPUT_FORMATS[0]
        dtformats['TIME'] = settings.TIME_INPUT_FORMATS[0]
        dtformats['SHORT_DATETIME'] = settings.DATETIME_INPUT_FORMATS[0]

        return dtformats


class TemplateView(View, base.TemplateView):
    pass


class DetailView(View, LinksMixin, base.DetailView):
    """
    Custom detail view.
    """

    fields = None  # Which fields should be shown in the details
    links = None   # Optional links such as viewing list of linked items

    def get_fields(self, obj):
        result = OrderedDict()
        if self.fields:
            for field_name in self.fields:
                if isinstance(field_name, tuple):
                    # custom property that is not a field of the model
                    result[field_name[1]] = getattr(obj, field_name[0])
                else:
                    field = self.model._meta.get_field(field_name)
                    result[field.verbose_name.title()] = getattr(obj,
                                                                 field_name)
        return result

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        context['fields'] = self.get_fields(context['object'])
        context['links'] = self.get_links()
        return context


class ListView(View, base.ListView):
    """
    Custom listview. Adding filter, sorting and display logic.
    """
    template_name = 'arctic/base_list.html'
    fields = None  # Which fields should be shown in listing
    filter_fields = []  # One on one maping to django-filter fields meta option
    search_fields = []
    ordering_fields = []  # Fields with ordering (subset of fields)
    default_ordering = []  # Default ordering, e.g. ['title', '-brand']
    action_links = []  # "Action" links on item level. For example "Edit"
    field_links = {}
    field_classes = {}
    tool_links = []   # Global links. For Example "Add object"
    prefix = ''  # Prefix for embedding multiple list views in detail view

    def get(self, request, *args, **kwargs):
        objects = self.get_object_list()
        context = self.get_context_data(object_list=objects)

        return self.render_to_response(context)

    def get_object_list(self):
        if self.filter_fields or self.search_fields:
            filterset_class = self.get_filterset_class()
            self.filterset = self.get_filterset(filterset_class)
            self.object_list = self.filterset.qs
        else:
            self.object_list = self.get_queryset()

        return self.object_list

    def ordering_url(self, field):
        """
        Creates a url link for sorting the given field.

        The direction of sorting will be either ascending, if the field is not
        yet sorted, or the opposite of the current sorting if sorted.
        """
        path = self.request.path
        direction = ''
        query_params = self.request.GET.copy()
        ordering = self.request.GET.get('order', '').split(',')
        if not ordering:
            ordering = self.get_default_ordering()
        merged_ordering = list(ordering)  # copy the list

        for ordering_field in self.ordering_fields:
            if (ordering_field.lstrip('-') not in ordering) and \
               (('-' + ordering_field.lstrip('-')) not in ordering):
                merged_ordering.append(ordering_field)

        new_ordering = []
        for item in merged_ordering:
            if item.lstrip('-') == field.lstrip('-'):
                if (item[0] == '-') or not (item in ordering):
                    if item in ordering:
                        direction = 'desc'
                    new_ordering.insert(0, item.lstrip('-'))
                else:
                    direction = 'asc'
                    new_ordering.insert(0, '-' + item)

        query_params['order'] = ','.join(new_ordering)

        return (path + '?' + query_params.urlencode(safe=','), direction)

    def get_fields(self):
        return self.fields

    def get_field_links(self):
        if not self.field_links:
            return {}
        else:
            allowed_field_links = {}
            for field, url in self.field_links.items():

                # check permission based on named_url
                if not view_from_url(url).has_permission(self.request.user):
                    continue

                allowed_field_links[field] = url
            return allowed_field_links

    def get_field_classes(self):
        return self.field_classes

    def get_list_header(self):
        """
        Creates a list of dictionaries with the field names, labels,
        field links, field css classes, order_url and order_direction,
        this simplifies the creation of a table in a template.
        """
        model = self.object_list.model
        result = []
        if not self.get_fields():
            result.append({
                'name': '',
                'verbose': str(model._meta.verbose_name),
            })
        else:
            prefix = self.get_prefix()
            for field_name in self.get_fields():
                item = {}
                if isinstance(field_name, tuple):
                    # custom property that is not a field of the model
                    name = field_name[0]
                    item['label'] = field_name[1]
                else:
                    name = field_name
                    try:
                        field_meta = find_field_meta(
                            model,
                            field_name
                        )
                        if field_meta._verbose_name:  # noqa
                            # explicitly set on the model, so don't change
                            item['label'] = field_meta._verbose_name  # noqa
                        else:
                            # title-case the field name (issue #80)
                            item['label'] = field_meta.verbose_name.title()

                        # item['label'] = model._meta.get_field(field_name).\
                        #     verbose_name
                    except FieldDoesNotExist:
                        item['label'] = field_name
                    except AttributeError:
                        item['label'] = field_name
                item['name'] = prefix + name
                if name in self.get_field_links().keys():
                    item['link'] = self.get_field_links()[name]
                if name in self.get_field_classes().keys():
                    item['class'] = self.get_field_classes()[name]
                if name in self.ordering_fields:
                    item['order_url'], item['order_direction'] = \
                        self.ordering_url(name)
                result.append(item)

        return result

    def get_list_items(self, objects):
        items = []
        if not self.fields:
            for obj in objects:
                items.append([obj.pk, str(obj)])
        else:
            for obj in objects:
                item = [get_attribute(obj, 'pk')]
                for field_name in self.fields:
                    if isinstance(field_name, tuple):
                        try:
                            virtual_field_name = "get_{}_field".\
                                format(field_name[0])
                            value = getattr(self, virtual_field_name)(obj)
                        except AttributeError:
                            value = find_attribute(obj, field_name[0])
                    else:
                        try:
                            # Get the choice display value
                            parent_objs = '__'.join(
                                field_name.split('__')[:-1])
                            method_name = 'get_{}_display'.format(
                                field_name.split('__')[-1])
                            value = find_attribute(obj, parent_objs + '__' +
                                                   method_name)()
                        except AttributeError:
                            try:
                                virtual_field_name = "get_{}_field".\
                                    format(field_name)
                                value = getattr(self, virtual_field_name)(obj)
                            except AttributeError:
                                value = find_attribute(obj, field_name)

                    item.append(value)
                items.append(item)
        return items

    def get_action_links(self):
        if not self.action_links:
            return []
        else:
            allowed_action_links = []
            for link in self.action_links:

                # check permission based on named_url
                if not view_from_url(link[1]).\
                        has_permission(self.request.user):
                    continue

                icon = None
                if len(link) == 3:  # if an icon class is given
                    icon = link[2]
                allowed_action_links.append({'label': link[0],
                                             'url': link[1],
                                             'icon': icon})
            return allowed_action_links

    def get_tool_links(self):
        if not self.tool_links:
            return []
        else:
            allowed_tool_links = []
            for link in self.tool_links:

                # check permission based on named_url
                if not view_from_url(link[1]).\
                        has_permission(self.request.user):
                    continue

                icon = None
                if len(link) == 3:  # if an icon class is given
                    icon = link[2]
                allowed_tool_links.append({'label': link[0],
                                           'url': link[1],
                                           'icon': icon})
            return allowed_tool_links

    def get_prefix(self):
        return self.prefix + '-' if self.prefix else ''

    def get_default_ordering(self):
        prefix = self.get_prefix()
        return [prefix + f for f in self.default_ordering]

    def get_ordering_with_prefix(self):
        return self.request.GET.getlist('order', self.get_default_ordering())

    def get_ordering(self):
        """Ordering used for queryset filtering (should not contain prefix)."""
        prefix = self.get_prefix()
        fields = self.get_ordering_with_prefix()
        if self.prefix:
            fields = [f.replace(prefix, '', 1) for f in fields]
        return [f for f in fields if f.lstrip('-') in self.ordering_fields]

    def get_filterset_class(self):
        if not self.filter_fields and not self.search_fields:
            return None

        return filterset_factory(self.model or self.queryset.model,
                                 self.filter_fields, self.search_fields)

    def get_filterset(self, filterset_class):
        """
        Returns an instance of the filterset to be used in this view.
        """
        kwargs = self.get_filterset_kwargs(filterset_class)
        return filterset_class(**kwargs)

    def get_filterset_kwargs(self, filterset_class):
        """
        Returns the keyword arguments for instanciating the filterset.
        """
        data = self.request.GET.copy()
        for key in self.request.GET:
            if not data[key]:
                data.pop(key)

        kwargs = {
            'data': data,
            'queryset': self.get_queryset(),
        }
        if self.prefix:
            kwargs['prefix'] = self.prefix
        return kwargs

    def get_page_title(self):
        if not self.page_title:
            return capfirst(self.object_list.model._meta.verbose_name_plural)
        return self.page_title

    def get_context_data(self, **kwargs):
        context = super(ListView, self).get_context_data(**kwargs)
        context['prefix'] = self.prefix
        context['list_header'] = self.get_list_header()
        context['list_items'] = self.get_list_items(context['object_list'])
        context['action_links'] = self.get_action_links()
        context['tool_links'] = self.get_tool_links()
        if self.filter_fields or self.search_fields:
            context['has_filter'] = True
            context['filter'] = self.filterset
        return context


class CreateView(View, SuccessMessageMixin, LayoutMixin, base.CreateView):
    template_name = 'arctic/base_create_update.html'
    success_message = _('%(object)s was created successfully')

    def get_page_title(self):
        if not self.page_title:
            return _("Create %s") % self.model._meta.verbose_name
        return self.page_title


class UpdateView(SuccessMessageMixin, LayoutMixin, View, LinksMixin,
                 extra_views.UpdateWithInlinesView):
    template_name = 'arctic/base_create_update.html'
    success_message = _('%(object)s was updated successfully')

    links = None  # Optional links such as viewing list of linked items

    def get_page_title(self):
        if not self.page_title:
            return _("Edit %s") % self.model._meta.verbose_name
        return self.page_title

    def get_context_data(self, **kwargs):
        context = super(UpdateView, self).get_context_data(**kwargs)
        context['links'] = self.get_links()
        context['layout'] = self.get_layout()
        return context


class FormView(View, SuccessMessageMixin, LayoutMixin, base.FormView):
    template_name = 'arctic/base_create_update.html'


class DeleteView(SuccessMessageMixin, View, base.DeleteView):
    template_name = 'arctic/base_confirm_delete.html'

    def get(self, request, *args, **kwargs):
        """
        Catch protected relations and show to user.
        """
        self.object = self.get_object()
        can_delete = True
        protected_objects = []
        collector_message = None
        collector = Collector(using='default')
        try:
            collector.collect([self.object])
        except ProtectedError as e:
            collector_message = "Cannot delete %s because it has relations " \
                                "that depends on it." % self.object
            protected_objects = e.protected_objects
            can_delete = False

        context = self.get_context_data(object=self.object,
                                        can_delete=can_delete,
                                        collector_message=collector_message,
                                        protected_objects=protected_objects)
        return self.render_to_response(context)


class LoginView(TemplateView):
    template_name = 'arctic/login.html'
    page_title = 'Login'
    requires_login = False

    def __init__(self, *args, **kwargs):
        super(TemplateView, self).__init__(*args, **kwargs)
        # thread-safe definition of messages.
        self.messages = []

    def get_context_data(self, **kwargs):
        # Call the base implementation first to get a context
        context = super(LoginView, self).get_context_data(**kwargs)
        context['next'] = self.request.GET.get('next', '/')
        context['username'] = self.request.POST.get('username', '')
        context['messages'] = set(self.messages)
        return context

    def get(self, request, *args, **kwargs):
        logout(request)
        return super(LoginView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        user = authenticate(username=request.POST['username'],
                            password=request.POST['password'])
        if user and user.is_active:
            login(request, user)
            return redirect(request.GET.get('next', '/'))

        self.messages.append('Invalid username/password combination')

        return render(request, self.template_name,
                      self.get_context_data(**kwargs))



from collections import OrderedDict
# import django.core.urlresolvers
# from django.http import HttpRequest
#
# from arctic import utils


class TestMenu:
    MENU = (
        ('menu 1', 'url_1', 'icon 1'),
        ('menu 2', 'url_2', 'icon 2', (
            ('menu 2_1', 'url_2_1'),
            ('menu 2_2', 'url_2_2'),
        )),
        ('menu 3', '/'),
        ('menu 4', 'url_4', (
            ('menu 4_1', 'url_4_1', 'icon 4_1'),
        )),
    )

    OUTPUT_MENU = OrderedDict([
        ('menu 1', {'url': 'url_1', 'icon': 'icon 1', 'active': False,
                    'active_weight': 5, 'submenu': None}),
        ('menu 2', {'url': 'url_2', 'icon': 'icon 2', 'active': False,
                    'active_weight': 5, 'submenu': OrderedDict([
            ('menu 2_1', {'url': 'url_2_1', 'icon': None, 'active': False,
                          'active_weight': 7, 'submenu': None}),
            ('menu 2_2', {'url': 'url_2_2', 'icon': None, 'active': False,
                          'active_weight': 7, 'submenu': None}),
        ])}),
        ('menu 3', {'url': '/', 'icon': None, 'active': True,
                    'active_weight': 1, 'submenu': None}),
        ('menu 4', {'url': 'url_4', 'icon': None, 'active': False,
                    'active_weight': 5, 'submenu': OrderedDict([
            ('menu 4_1', {'url': 'url_4_1', 'icon': 'icon 4_1',
                          'active': False, 'active_weight': 7,
                          'submenu': None}),
        ])}),
    ])  # noqa

    # def test_menu(self, monkeypatch):
    #     """
    #     Test menu items with and without icons and with and without submenus
    #     """
    #     request = HttpRequest()
    #     request.path = '/'
    #     kwargs = {'request': request}
    #
    #     monkeypatch.setattr(django.core.urlresolvers, 'reverse',
    #                         lambda url: url)
    #     menu = utils.menu(menu_config=self.MENU, **kwargs)
    #
    #     assert menu == self.OUTPUT_MENU



ARCTIC_SITE_NAME = 'Arctic Example'

# Menu format:
# (('menu label', 'named url', 'optional icon class', (optional submenu)))
ARCTIC_MENU = (
    ('Dashboard', 'index', 'fa-dashboard'),
    ('Articles', 'articles:list', 'fa-file-text-o', (
        ('List', 'articles:list'),
        ('Create', 'articles:create'),
    )),
    ('Categories', 'articles:category-list', 'fa-sitemap', (
        ('List', 'articles:category-list'),
        ('Create', 'articles:category-create'),
    )),
    ('Tags', 'articles:tag-list', 'fa-tags', (
        ('List', 'articles:tag-list'),
        ('Create', 'articles:tag-create'),
    )),
    ('Users', 'users:list', 'fa-user', (
        ('List', 'users:list'),
        ('Create', 'users:create'),
    )),
)

ARCTIC_USER_ROLE_MODEL = 'arctic.UserRole'
ARCTIC_ROLE_MODEL = 'arctic.Role'

ARCTIC_ROLES = {
    'editor': (
        'articles_view',
    ),
}

# ARCTIC_TOPBAR_BACKGROUND_COLOR = '#BF1D1D'
# ARCTIC_HIGHLIGHT_COLOR = '#9ec3d5'

# ARCTIC_TOPBAR_BACKGROUND_COLOR = '#212050'
# ARCTIC_HIGHLIGHT_COLOR = '#76a2cb'



from django.conf import settings
from django.db import models


class UserRole(models.Model):
    user = models.OneToOneField(settings.AUTH_USER_MODEL, related_name='urole')
    role = models.ForeignKey('arctic.Role')

    class Meta:
        swappable = 'ARCTIC_USER_ROLE_MODEL'


class Role(models.Model):
    name = models.CharField('Role', max_length=100, unique=True)
    is_active = models.BooleanField(default=True)

    def __str__(self):
        return self.name

    class Meta:
        swappable = 'ARCTIC_ROLE_MODEL'



import pytest
from django.core.exceptions import PermissionDenied
from django.test import Client


class TestCMS(object):
    """
    Tests that probe the CMS with the Django test client
    """

    @pytest.fixture(scope="session", autouse=True)
    def init(self):
        self.__class__.client = Client()

    def test_login_success(self, admin_user):
        """
        Asserts that an admin user can log in an see a restricted page
        within the CMS

        :param admin_user: fixture provided by pytest-django
        :return: void
        """
        self.client.post('/login/',
                         {
                             'username': 'admin',
                             'password': 'password'
                         })
        response = self.client.get('/articles/')

        assert response.status_code == 200

    def test_login_fail(self, admin_user):
        """
        Asserts that an unauthorized user cannot see a restricted page

        :return:
        """
        try:
            response = self.client.get('/articles/')
            assert response.status_code != 200
        except PermissionDenied:
            assert True



from arctic.generics import TemplateView


class BadRequestView(TemplateView):
    page_title = 'Bad Request'
    template_name = 'arctic/400.html'
    permission_required = ""

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context, status=400)


class ForbiddenView(TemplateView):
    page_title = 'Access Forbidden'
    template_name = 'arctic/403.html'
    permission_required = ""

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context, status=403)


class NotFoundView(TemplateView):
    page_title = 'Page not Found'
    template_name = 'arctic/404.html'
    permission_required = ""

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context, status=404)


class InternalErrorView(TemplateView):
    page_title = 'Internal Error'
    template_name = 'arctic/500.html'
    permission_required = ""

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context, status=500)

handler400 = BadRequestView.as_view()
handler403 = ForbiddenView.as_view()
handler404 = NotFoundView.as_view()
handler500 = InternalErrorView.as_view()



# from django.db import models

# import pytest
# from mixer.backend.django import mixer
# pytestmark = pytest.mark.django_db


# class Book(models.Model):
#     title = models.CharField("Title", max_length=255, null=False)
#     description = models.TextField("Description", blank=True, null=False)
#     category = models.ForeignKey('tests.Category')
#     tags = models.ManyToManyField('tests.Tag')


# class Category(models.Model):
#     name = models.CharField('Name', max_length=255, null=False, blank=False,
#                             unique=True)

#     def __str__(self):
#         return self.name


# class Tag(models.Model):
#     name = models.CharField('Tag', max_length=255, null=False, blank=False,
#                             unique=True)

#     def __str__(self):
#         return self.name



from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse_lazy
from django.utils.translation import ugettext as _

from arctic.generics import (CreateView, ListView, UpdateView)
from arctic.loading import get_user_role_model

from .forms import (UserChangeMultiForm, UserCreationMultiForm)

User = get_user_model()
UserRole = get_user_role_model()


class UserListView(ListView):
    page_title = _('Users')
    paginate_by = 20
    model = UserRole
    fields = [('user__username', 'Username'), 'role__name', 'user__is_active',
              'user__last_login']
    ordering_fields = ['user__username', 'role__name', 'user__last_login']
    search_fields = ['user__username']
    filter_fields = ['user__is_active']
    # action_links = [
    #     ('delete', 'users:delete', 'fa-trash'),
    # ]
    field_links = {
        'user__username': 'users:detail',
    }
    tool_links = [
        (_('Create Users'), 'users:create', 'fa-plus'),
    ]
    permission_required = ""


class UserCreateView(CreateView):
    page_title = _('Create User')
    model = UserRole
    success_url = reverse_lazy('users:list')
    form_class = UserCreationMultiForm
    permission_required = ""

    def get_success_message(self, cleaned_data):
        return _('User {} was successfully created').format(
            self.object['user'])


class UserUpdateView(UpdateView):
    page_title = _('Change User')
    model = UserRole
    success_url = reverse_lazy('users:list')
    form_class = UserChangeMultiForm
    permission_required = ""

    def get_form_kwargs(self):
        kwargs = super(UserUpdateView, self).get_form_kwargs()
        kwargs.update(instance={
            'user': self.object.user,
            'role': self.object
        })
        return kwargs

    def get_success_message(self, cleaned_data):
        return _('User {} was successfully updated').format(
            self.object['user'])



import pytest

from arctic.generics import ListView
from .conftest import article


@pytest.fixture
def virtual_fields_example_1():
    class TestListView(ListView):
        paginate_by = 2
        fields = ['title', 'description', 'published', 'content_type']
        permission_required = "articles_view"
        queryset = article()

    return TestListView()


def test_virtual_fields_example_1(article, virtual_fields_example_1):
    list_view = virtual_fields_example_1

    with pytest.raises(AttributeError) as excinfo:
        list_view.get_list_items([article])

    message = "'Article' object has no attribute 'content_type'"
    assert str(excinfo.value) == message


@pytest.fixture
def virtual_fields_example_2():
    class TestListView(ListView):
        paginate_by = 2
        fields = ['title', 'description', 'published', 'virtual_field']
        permission_required = "articles_view"
        queryset = article()

        def get_virtual_field_field(self, row):
            return 'Virtual Field: ' + row.title

    return TestListView()


def test_virtual_fields_example_2(article, virtual_fields_example_2):
    list_view = virtual_fields_example_2
    res = list_view.get_list_items([article])

    assert res[0][0] is None
    assert res[0][1] == 'title1'
    assert res[0][2] == 'description1'
    assert res[0][3] == 'published1'
    assert res[0][4] == 'Virtual Field: title1'


@pytest.fixture
def virtual_fields_example_3():
    class TestListView(ListView):
        paginate_by = 2
        fields = ['title', 'description', 'published', 'virtual_field']
        permission_required = "articles_view"
        queryset = article()

    return TestListView()


def test_virtual_fields_example_3(article, virtual_fields_example_3):
    list_view = virtual_fields_example_3

    with pytest.raises(AttributeError) as excinfo:
        list_view.get_list_items([article])

    message = "'Article' object has no attribute 'virtual_field'"
    assert str(excinfo.value) == message



from __future__ import (absolute_import, unicode_literals)

import operator
from functools import reduce

from django import forms
from django.db.models import (DateTimeField, Q, BooleanField)

import django_filters
from django_filters.widgets import RangeWidget


class SearchInput(forms.TextInput):
    """ Normal text input with search flag.
    Just so the frontend knows it's a search box
    """
    is_search = True


class DateRangeInput(RangeWidget):
    """ Range widget used for range date input.
    Just so the frontend knows it's a date range box
    """
    pass


class BooleanFilterSelect(forms.NullBooleanSelect):
    """ Yes/No select widget for boolean select field. """
    def render(self, name, value, attrs=None):
        for option_value, option_label in enumerate(self.choices):
            if option_label[0] == '1':
                self.choices[option_value] = (option_label[0], name)
            else:
                label = name + ': ' + str(option_label[1])
                self.choices[option_value] = (option_label[0], label)

        return super(BooleanFilterSelect, self).render(name, value, attrs)


class FilterSet(django_filters.FilterSet):
    filter_overrides = {
        DateTimeField: {
            'filter_class': django_filters.DateFromToRangeFilter,
            'extra': lambda f: {
                'widget': DateRangeInput,
            }
        },
        BooleanField: {
            'filter_class': django_filters.BooleanFilter,
            'extra': lambda f: {
                'widget': BooleanFilterSelect,
            }
        }
    }
    search_fields = None

    def __init__(self, *args, **kwargs):
        super(FilterSet, self).__init__(*args, **kwargs)
        for filter_name in self.filters:
            """ We want a descriptive title for selects """
            if isinstance(self.filters[filter_name],
                          django_filters.ModelChoiceFilter):
                self.filters[filter_name].extra.update(
                    {'empty_label': u"%s" % self.filters[filter_name].label}
                )
            elif isinstance(self.filters[filter_name],
                            django_filters.ChoiceFilter):
                choices = (('', self.filters[filter_name].label),) + \
                    self.filters[filter_name].extra['choices']
                self.filters[filter_name].extra.update(
                    {'choices': choices}
                )

    def search_filter(self, queryset, value):
        if self.search_fields:
            q_list = []
            for field_name in self.search_fields:
                q_list.append(Q(**{field_name + '__icontains': value}))

            queryset = queryset.filter(reduce(operator.or_, q_list))

        return queryset


def filterset_factory(model, fields, search_fields=None):
    if search_fields:
        fields = list(fields) + ['search']

    class DynamicFilterSet(FilterSet):
        if search_fields:
            search = django_filters.MethodFilter(action='search_filter',
                                                 widget=SearchInput)

    meta = type(str('Meta'), (object,), {
        'model': model,
        'fields': fields,
    })
    filterset = type(
        str('%sFilterSet' % model._meta.object_name), (DynamicFilterSet,), {
            'Meta': meta,
            'search_fields': search_fields,
        }
    )

    return filterset



from setuptools import (find_packages, setup)


__VERSION__ = '0.9.3'

try:
    README = open('README.md').read()
except:
    README = None

try:
    REQUIREMENTS = open('requirements/base.txt').read()
except:
    REQUIREMENTS = None

setup(
    name='django-arctic',
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Internet :: WWW/HTTP',
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    version=__VERSION__,
    long_description=README,
    install_requires=REQUIREMENTS,
    packages=find_packages(),
    include_package_data=True,
    license='MIT',
    url='https://github.com/sanoma/django-arctic',
    author='Sanoma Netherlands',
    author_email='opensource@sanoma.com',
    description="Django framework to create custom content management systems",
)



# -*-*- encoding: utf-8 -*-*-
from __future__ import unicode_literals

from django.db import models


class Article(models.Model):
    created_at = models.DateTimeField(auto_now=True)
    updated_at = models.DateTimeField()
    title = models.CharField("Title", max_length=255, null=False)
    description = models.TextField("Description", blank=True, null=False)
    published = models.BooleanField("Published", default=False)
    category = models.ForeignKey('articles.Category', verbose_name="Category")
    tags = models.ManyToManyField('articles.Tag')

    def __str__(self):
        return 'Article "' + self.title + '"'


class Category(models.Model):
    name = models.CharField('Name', max_length=255, null=False, blank=False,
                            unique=True)

    def __str__(self):
        return self.name


class Tag(models.Model):
    term = models.CharField('Term', max_length=255, null=False, blank=False,
                            unique=True)

    def __str__(self):
        return self.term
